/*
 * Initial main.c file generated by Glade. Edit as required.
 * Glade will not overwrite this file.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>
              
#include "main.h"
#include "funcs.h"
#include "interface.h"
#include "support.h"
#include "callbacks.h"
#include "db.h"

static GtkWidget *image_drawingarea;
static LIST_HEAD(thumbnails_anchor);

//static GdkInterpType interp_type = GDK_INTERP_NEAREST;
static GdkInterpType interp_type = GDK_INTERP_TILES;
//static GdkInterpType interp_type = GDK_INTERP_BILINEAR;
//static GdkInterpType interp_type = GDK_INTERP_HYPER

//static GdkRgbDither dither_type = GDK_RGB_DITHER_NONE;
static GdkRgbDither dither_type = GDK_RGB_DITHER_NORMAL;
//static GdkRgbDither dither_type = GDK_RGB_DITHER_MAX;
static GdkPixbuf *image_pixbuf = NULL;

#define THUMB_HEIGHT 96

static gboolean image_drawingarea_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
	GdkPixbuf *dest;
	double scale_factor,scale_width,scale_height;
	unsigned int image_width,image_height;
	unsigned int drawable_width,drawable_height;

	if (!image_pixbuf)
		return TRUE;

	gdk_window_set_back_pixmap(widget->window, NULL, FALSE);

	dest = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, event->area.width, event->area.height);

	image_width = gdk_pixbuf_get_width(image_pixbuf);
	image_height = gdk_pixbuf_get_height(image_pixbuf);
	drawable_width = widget->allocation.width;
	drawable_height = widget->allocation.height;
	scale_width = (double)drawable_width / image_width;
	scale_height = (double)drawable_height / image_height;
	if (scale_width < scale_height)
		scale_factor = scale_width;
	else
		scale_factor = scale_height;

fprintf(stderr,"Expose event: x:%u y:%u width:%u height:%u\n",event->area.x,event->area.y,event->area.width,event->area.height);
#if 1
fprintf(stderr,"Drawing: %p ... %p\n",image_pixbuf,dest);
	gdk_pixbuf_scale(image_pixbuf,dest,
		0,0,event->area.width,event->area.height,
		-event->area.x,-event->area.y,
		scale_factor,scale_factor,
		interp_type);
#else
	gdk_pixbuf_composite_color(image_pixbuf,dest,
		0,0,event->area.width,event->area.height,
		-event->area.x,-event->area.y,
		scale_factor,scale_factor,
		interp_type,
		255,
		event->area.x,event->area.y,16,0xaaaaaa,0x555555);
#endif
fprintf(stderr,"rendering..\n");
	gdk_pixbuf_render_to_drawable (dest, widget->window,
		widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
		0, 0, event->area.x,event->area.y,
		event->area.width,event->area.height,
		dither_type,event->area.x,event->area.y);

	gdk_pixbuf_unref(dest);

	return TRUE;
}

static int maximum_thumbnail_width = 0;

static void load_thumbnail_image(const char *file,GtkWidget *image_clist,int row){
	GdkPixbuf *pixbuf,*pbscaled;
	GdkPixmap *pixmap;
	GdkBitmap *mask;
	unsigned int width, height;

	pixbuf = gdk_pixbuf_new_from_file(file);

	width = gdk_pixbuf_get_width(pixbuf);
	height = gdk_pixbuf_get_height(pixbuf);
	if (width > maximum_thumbnail_width){
		maximum_thumbnail_width = width;
		gtk_clist_set_column_width(GTK_CLIST(image_clist), 0, maximum_thumbnail_width);
	}
//	width = width * THUMB_HEIGHT / height;
//	height = THUMB_HEIGHT;
	fprintf(stderr,"width:%u height:%u\n",width,height);

//	pbscaled = gdk_pixbuf_scale_simple(pixbuf,width,height,GDK_INTERP_BILINEAR);
//	gdk_pixbuf_unref(pixbuf);
	pbscaled = pixbuf;

	gdk_pixbuf_render_pixmap_and_mask(pbscaled, &pixmap, &mask, 128);
	gdk_pixbuf_unref(pbscaled);

//	gtk_clist_set_shift(GTK_CLIST(image_clist), row, 0, 0, 0);

	gtk_clist_set_pixmap(GTK_CLIST(image_clist),row,0,pixmap,mask);
}

void load_full_image(struct image_data *idp){

	g_assert(idp);

	if (image_pixbuf)
		gdk_pixbuf_unref(image_pixbuf);

	image_pixbuf = gdk_pixbuf_new_from_file(idp->fullsize_path);

	gtk_widget_queue_draw(image_drawingarea);
}

static void thumbnails_destroy(struct list_head *anchor){
	struct list_head *lp,*nextlp;
	struct image_data *idp;

	for (lp = anchor->next;lp != anchor;lp = nextlp){
		nextlp = lp->next;
		idp = list_entry(lp,struct image_data,list);

		free(idp->name);
		free(idp->thumbnail_path);
		free(idp->fullsize_path);
		free_structure(idp);
	}
	INIT_LIST_HEAD(anchor);
}

static void update_thumbs(GtkWidget *image_clist,struct db_handle *db){
	struct db_result *res;
	struct image_data *idp;
	char **row;
	gchar *clist_row[3+1];
	int rownum;

	if (db_query(db,"select name,"
			"concat('/foo/raid/pictures/web/chris_sim/thumbnails/',path), "
			"concat('/foo/raid/pictures/web/chris_sim/original/',path), "
			"display "
			"from image_data where time > '2002-11-01'") != 0)
		return;

	gtk_clist_clear(GTK_CLIST(image_clist));
	thumbnails_destroy(&thumbnails_anchor);
	maximum_thumbnail_width = 0;
	gtk_clist_set_row_height(GTK_CLIST(image_clist), THUMB_HEIGHT + 2);

	res = db_use_result(db);
	while ((row = db_fetch_row(res)) != NULL){
		if (row[0] && row[1]){
			idp = alloc_structure(struct image_data);
			idp->name = safe_strdup(row[0]);
			idp->thumbnail_path = safe_strdup(row[1]);
			idp->fullsize_path = safe_strdup(row[2]);
			list_add_tail(&idp->list,&thumbnails_anchor);

			clist_row[0] = idp->name;
			clist_row[1] = (row[2] && atoi(row[2])) ? "Show" : "Hide";
			clist_row[2] = idp->name;
			clist_row[3] = NULL;
			rownum = gtk_clist_append(GTK_CLIST(image_clist),clist_row);
			gtk_clist_set_row_data(GTK_CLIST(image_clist),rownum,idp);
			load_thumbnail_image(idp->thumbnail_path,image_clist,rownum);
		}
	}
	db_free_result(res);
}

int
main (int argc, char *argv[])
{
	GtkWidget *editor_window;
	GtkWidget *hscale;
	GtkWidget *image_clist;
	struct db_handle *db;
	struct db_result *res;
	char **row;
	int min_year,max_year;

	db = db_connect();
	if (db == NULL)
		exit(1);

	if (db_query(db,"select min(year(time)),max(year(time)) from image_data") != 0)
		exit(1);

	min_year = 2000;
	max_year = 2020;
	res = db_use_result(db);
	while ((row = db_fetch_row(res)) != NULL){
		if (row[0] && row[1]){
			min_year = atoi(row[0]);
			max_year = atoi(row[1]);
		}
	}
	db_free_result(res);

//	gtk_set_locale();
	gtk_init(&argc, &argv);

	add_pixmap_directory(PACKAGE_DATA_DIR "/pixmaps");
	add_pixmap_directory(PACKAGE_SOURCE_DIR "/pixmaps");

	/*
	 * The following code was added by Glade to create one of each component
	 * (except popup menus), just so that you see something after building
	 * the project. Delete any components that you don't want shown initially.
	 */
	editor_window = create_editor_window();
	hscale = lookup_widget(editor_window,"year_hscale");
	gtk_signal_connect(GTK_OBJECT (GTK_RANGE (hscale)->adjustment),
		"value_changed",GTK_SIGNAL_FUNC(on_change_year_hscale),NULL);
	gtk_adjustment_clamp_page(GTK_ADJUSTMENT(GTK_RANGE(hscale)->adjustment),
		min_year,max_year);
	hscale = lookup_widget(editor_window,"month_hscale");
	gtk_signal_connect(GTK_OBJECT (GTK_RANGE (hscale)->adjustment),
		"value_changed",GTK_SIGNAL_FUNC(on_change_month_hscale),NULL);
	gtk_widget_show(editor_window);

	image_clist = lookup_widget(editor_window,"image_clist");
	update_thumbs(image_clist,db);

	image_drawingarea = lookup_widget(editor_window,"image_drawingarea");
	gtk_signal_connect(GTK_OBJECT(image_drawingarea), "expose_event",
		GTK_SIGNAL_FUNC(image_drawingarea_expose_event),NULL);

	gtk_main ();
	thumbnails_destroy(&thumbnails_anchor);
	return 0;
}

